import org.drools.KnowledgeBase
import org.drools.KnowledgeBaseFactory
import org.drools.builder.KnowledgeBuilder
import org.drools.builder.KnowledgeBuilderFactory
import org.drools.builder.ResourceType
import org.drools.definition.KnowledgePackage
import org.drools.io.ResourceFactory
import org.drools.runtime.StatefulKnowledgeSession
import pokie.Evaluation

public class EvaluatorService {
  static final RULES = 'pokie.drl'
  static transactional = false

//  static final Logger log = Logger.getLogger(Evaluator.class)

//  static evaluate = {spindle ->
//    if (!spindle.initialised)
//    return EvaluatorResult.INACTIVE;
//
//    def firstVal = spindle.leftReel.value
//    if (spindle.find {k, v -> v.value != firstVal} != null)
//      return EvaluatorResult.LOSE
//
//    return firstVal == 3 ? EvaluatorResult.BIGWIN : EvaluatorResult.SMALLWIN
//  }


  def evaluate = {spindle ->

    def kbuilder = KnowledgeBuilderFactory.newKnowledgeBuilder();

// this will parse and compile in one step
    kbuilder.add(ResourceFactory.newClassPathResource(RULES,
            EvaluatorService.class), ResourceType.DRL);

// Check the builder for errors
    if (kbuilder.hasErrors()) {
      def errs = kbuilder.getErrors().toString()
      def msg = "Unable to compile '${RULES}': ${errs}"
      log.error (msg);
      throw new RuntimeException(msg);
    }

// get the compiled packages (which are serializable)
    def pkgs = kbuilder.getKnowledgePackages();

// add the packages to a knowledgebase (deploy the knowledge packages).
    def kbase = KnowledgeBaseFactory.newKnowledgeBase();
    kbase.addKnowledgePackages(pkgs);

    def ksession = kbase.newStatefulKnowledgeSession();

    def e = new Evaluation(leftReelValue: spindle.spindle.leftReel.value,
            middleReelValue: spindle.spindle.middleReel.value, rightReelValue: spindle.spindle.rightReel.value, initialised: spindle.initialised)
    ksession.insert(e);

    ksession.fireAllRules();

    ksession.dispose();

    e.getResult()
  }
}
